package com.link.core.basic.database;

import com.link.base.user.model.CoreUser;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import java.util.Random;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class DynamicDataSourceHolder {
   private static final Logger logger = LogManager.getLogger(DynamicDataSourceHolder.class);
   public static final ThreadLocal<String> holder = new ThreadLocal();
   private static final DataSourceType[] randomDataSourceArr;

   public static void putDataSource(String name) {
      if (StringUtils.equals(name, DataSourceType.RANDOM.getName())) {
         name = getUserRandomSlave(UserUtil.getUser());
      }

      holder.set(name);
   }

   public static String getDataSouce() {
      String holderValue = (String)holder.get();
      if (StringUtils.isBlank(holderValue)) {
         holderValue = DataSourceType.MASTER.getName();
      }

      return holderValue;
   }

   public static void clearDataSouce() {
      holder.remove();
   }

   public static String getUserRandomSlave(CoreUser user) {
      String defaultRandomDataSourceName = DataSourceType.MASTER.getName();
      if (user == null) {
         return defaultRandomDataSourceName;
      } else {
         String userRandomDataSourceName = user.getRandomDataSourceName();
         if (StringUtils.isBlank(userRandomDataSourceName)) {
            userRandomDataSourceName = defaultRandomDataSourceName;
         }

         return userRandomDataSourceName;
      }
   }

   public static DataSourceType getRandomSlave(DataSourceType[] slaveDataSourceArr) {
      int number = (new Random()).nextInt(slaveDataSourceArr.length);
      return slaveDataSourceArr[number];
   }

   public static DataSourceType[] getRandomDataSourceArr() {
      return randomDataSourceArr;
   }

   static {
      randomDataSourceArr = new DataSourceType[]{DataSourceType.MASTER, DataSourceType.SLAVE_01, DataSourceType.SLAVE_02};
   }
}
